// RBC_Model_ML.mod, version 3.1, fabio canova
// estimation of a RBC model with utility in leisure a-la Hansen (1985) via maximum likelihood 
// The model is estimated with either simulated or actual data; 
 
%***************L********L********L********L
%Loading the data to be used in estimation        
%****************L********L********L********L
estimation_dataset= input('Which dataset? Simulated = 1; True data = 2 '); % Select the Dataset

if estimation_dataset==1
          %  use simulated data generated with RBC_model_Simul.mod
load RBC_simuldata
save rbc_data1 iy nn ynr
else
          % use true data
load RBC_data
%plot(ynr)
bp=[100, 130];   % this defines break points in the trend
ynr=detrend(ynr,'linear',bp)
figure
plot(ynr)
nn=nn;
save rbc_data1 iy nn ynr
end

var y c i r k n z ynr nn iy;
 
varexo e u1 u2 u3;  
 
parameters yk_bar R_bar ik cy delta alfa eta rho sig_e;

N_bar     = 1.0/3; % Steady state employment is one fourth of total time endowment
Z_bar     = 1.0;   % Normalization
alfa       = 0.36; % Capital share
delta     = 0.025; % Depreciation rate for capital
R_bar     = 1.01;  % One percent real interest per quarter
eta       = 1.0;   % constant of relative risk aversion = 1/(coeff. of intertemporal substitution)
rho       = 0.95;  % autocorrelation of technology shock
sig_e  = 0.712;  % Standard deviation of technology shock.  Units: Percent.

// Steady states:
betta   = 1.0/R_bar;  % Discount factor beta
yk_bar  = (R_bar + delta - 1)/alfa;  
K_bar   = (yk_bar / Z_bar)^(1.0/(alfa-1)) * N_bar;
I_bar   = delta * K_bar;
Y_bar   = yk_bar * K_bar;
C_bar   = Y_bar - delta*K_bar;
A       =  C_bar^(-eta) * (1 - alfa) * Y_bar/N_bar; % Parameter in utility function
// parameters in the dynamic model  
ik = I_bar/K_bar;
cy = C_bar/Y_bar;
  
% We add three measurement errors to the observables yn (labor productivity) nn (employment), iy (investment 
% to output ratio)  that  is
% x(t) = A(theta)x(t-1)+B(theta) e(t)
% y(t) =  Cx(t) + u(t)
% where y(t) are the observable counterparts of the model based quantities x(t)
% and u(t) is a measurement error.

model(linear); 
% models
     y = cy*c+(1-cy)*i;
     k = (1-delta)*k(-1) + ik*i;
     y = alfa*k(-1) + (1-alfa)*n + z;
     n = -eta* c + y;
     r = - (alfa/R_bar)*yk_bar*k(-1) + (alfa/R_bar)*yk_bar*y;
     c = c(+1) - (1/eta)* r(+1);
     z = rho* z(-1) + e;
% observables
     ynr = y-n+u1;
     nn = n +u2;
     iy = i-y+u3;  
end; 

shocks;
var e; stderr sig_e;
var u1; stderr 0.7;  % measurement error 1
var u2; stderr 0.7;  % measurement error 2
var u3; stderr 0.7;  % measurement error 3
end;
 

estimated_params;
// old structure: PARAM NAME, INITIAL VALUE, LOWER BOUND, UPPER BOUND
stderr e, 0.4,0.1,1.9;
stderr u1,0.01,0.00001,2.9;
stderr u2,0.01,0.00001,2.9;
stderr u3,0.01,0.000001,2.9;                
rho,0.90,0.2,0.9999999;
alfa,0.4,0.05,0.75;
delta,0.025,0.015,0.15;
eta, 2.0,0.5,12.0;
cy,0.8,0.4,0.99;
end;


 varobs ynr nn iy;


 
// basic estimation command (ML)
% default maximization is mode_compute=4 (csminwel.m), alternatively use mode_compute=1 or
% mode_ compute=5. Careful with sevela options Hessian at the mode is not positive definite
% if you use actual data - no proble with simulated data

estimation(datafile=rbc_data1,mode_compute=5, mode_check) ynr nn iy;


%%%%%%% HINTS %%%%%%%
% careful with the boundaries. If estimates on the boundaries standard error not meaningful (t-stat=infinity)
% if you keep on hitting the boundaries fix these parameters and take them out of the optimization
% careful with mode computing routine. If you get non-positive definite hessian you can change routine
% or change the initial conditions.
% if nothing works look at your data, you may have hidden non-stationarities (see e.g. yn). In that
% case it is better to change observables.
% if you are still stuck and can't figure out what  to do to solve the problem with the non-positive 
% definite Hessian see RBC_model_ML_V2.mod for the solution.
%%%%%%%%%%%%%%%%%%%%%%%%


%% exercises
%% 1) add a goverment expenditure shock (AR1) , i.e. change line 35, add gy among the parameters and add g among the shocks
%% 2) add a fiscal rule ( g(t)= rhog g(t-1)+ ga n(t) + eg(t)). test hypothesis ga=0.
%% 3) with the estimated parameters compute impulse responses
 