// RBC_model_ML_V2.mod, version 2.0, fabio canova
// estimation of a RBC model with utility in leisure a-la Hansen (1985) via maximum likelihood 
// This is the same as RBC_model_ML.mod except that only a subset of the parameters are estimated.
// In particular, cy, alfa, delta and std measurement error3 are  not estimated.

close all; clc;


%*********L********L********L********L
% Loading the data        
%*********L********L********L********L

estimation_dataset= input('Which dataset? Simulated = 1; True data = 2 '); % Select the Dataset

if estimation_dataset==1
% simulated data generated with RBC_model_Simul_L.mod
load RBC_simuldata
save rbc_data1 iy nn ynr
else
% true data
load RBC_data
plot(ynr)
bp=[100, 130];
ynr=detrend(ynr,'linear',bp)
figure
plot(ynr)
nn=nn;
save rbc_data1 iy nn ynr
end

var y c i r k n z ynr nn iy;
 
varexo e u1 u2 u3;  
 
parameters yk_bar R_bar ik cy delta alfa eta rho sig_e;

N_bar     = 1.0/3; % Steady state employment is one fourth of total time endowment
Z_bar     = 1.0;   % Normalization
alfa       = 0.36; % Capital share
delta     = 0.025; % Depreciation rate for capital
R_bar     = 1.01;  % One percent real interest per quarter
eta       = 1.0;   % constant of relative risk aversion = 1/(coeff. of intertemporal substitution)
rho       = 0.95;  % autocorrelation of technology shock
sig_e  = 0.712;  % Standard deviation of technology shock.  Units: Percent.

// Steady states:
betta   = 1.0/R_bar;  % Discount factor beta
yk_bar  = (R_bar + delta - 1)/alfa;  
K_bar   = (yk_bar / Z_bar)^(1.0/(alfa-1)) * N_bar;
I_bar   = delta * K_bar;
Y_bar   = yk_bar * K_bar;
C_bar   = Y_bar - delta*K_bar;
A       =  C_bar^(-eta) * (1 - alfa) * Y_bar/N_bar; % Parameter in utility function
// parameters in the dynamic model  
ik = I_bar/K_bar;
cy = C_bar/Y_bar;
  
% model: we adding three measurement errors to the observables yn (labor productivity) nn (employment), iy (investment 
% to output ratio)  that  is
% x(t) = A(theta)x(t-1)+B(theta) e(t)
% y(t) =  Cx(t) + u(t)
% where y(t) are the observable counterparts of the model based quantities x(t)
% and u(t) is a measurement error.

model(linear); 
% model
     y = cy*c+(1-cy)*i;
     k = (1-delta)*k(-1) + ik*i;
     y = alfa*k(-1) + (1-alfa)*n + z;
     n = -eta* c + y;
     r = - (alfa/R_bar)*yk_bar*k(-1) + (alfa/R_bar)*yk_bar*y;
     c = c(+1) - (1/eta)* r(+1);
     z = rho* z(-1) + e;
% observables
     ynr = y-n+u1;
     nn = n +u2;
     iy = i-y+u3;  
end; 

shocks;
var e; stderr sig_e;
var u1; stderr 0.7;  % measurement error 1
var u2; stderr 0.7;  % measurement error 2
var u3; stderr 0.7;  % measurement error 3
end;
 

estimated_params;
// old structure: PARAM NAME, INITIAL VALUE, LOWER BOUND, UPPER BOUND
stderr e, 0.4,0.1,3.9;
stderr u1,0.7,0.00001,3.9;
stderr u2,0.7,0.00001,3.9;
//stderr u3,0.7,0.000001,3.9;                
rho,0.90,0.2,0.9999999;
//alfa,0.4,0.05,0.75;
//delta,0.025,0.015,0.15;
eta, 2.0,0.5,12.0;
//cy,0.8,0.4,0.99;


end;

 
 varobs ynr nn iy;


 
// basic estimation command (ML)
% default maximization is mode_compute=4 (csminwel.m), alternatively use mode_compute=1 or
% mode_ compute=5. Careful with some options Hessian at the mode is not positive definite

estimation(datafile=rbc_data1,mode_compute=5, mode_check) ynr nn iy;



   
  