// RBC MODEL solution: solves the three exercises in RBC Model ML 

%% exercises
%% 1) add a goverment expenditure shock (AR1)
%% 2) add a fiscal rule ( g(t)= rhog g(t-1)+ ga n(t) + eg(t)). test hypothesis ga=0.
%% 3) relax lower and/or upper bounds in estimation 

var y c i r k n z ynr nn iy g;
 
varexo e eg u1 u2 u3;  
 
parameters yk_bar R_bar ik cy delta alfa eta rho gy rhog ag;

N_bar     = 1.0/3; % Steady state employment is a third of total time endowment
Z_bar     = 1.0;   % Normalization
alfa       = 0.36; % Capital share
delta     = 0.025; % Depreciation rate for capital
R_bar     = 1.01;  % One percent real interest per quarter
eta       = 1.0;   % constant of relative risk aversion = 1/(coeff. of intertemporal substitution)
rho       = 0.95;  % autocorrelation of technology shock
sig_e  = 0.712;  % Standard deviation of technology shock.  Units: Percent.

// for exercises 1 and 2 add
 sig_g = 0.542;

// for exercise 2 also add 
%ag     = 0.2;

// Steady states:
betta   = 1.0/R_bar;  % Discount factor beta
yk_bar  = (R_bar + delta - 1)/alfa;  
K_bar   = (yk_bar / Z_bar)^(1.0/(alfa-1)) * N_bar;
I_bar   = delta * K_bar;
Y_bar   = yk_bar * K_bar;
C_bar   = Y_bar - delta*K_bar;
A       =  C_bar^(-eta) * (1 - alfa) * Y_bar/N_bar; % Parameter in utility function
// parameters in the dynamic model  
cy = C_bar/Y_bar;
ik = I_bar/K_bar;
  

// for exercises 1 and 2 add
 gy= 0.2;
 rhog = 0.80;

model(linear); 
// for exercise 1 use
 y= cy*c+gy*g+(1-cy-gy)*i;
 g = rhog*g(-1)+ eg;

// for exercise 2 use
% y= cy*c+gy*g+(1-cy-gy)*i;
% g = rhog*g(-1)+ ag*n+eg;

     k = (1-delta)*k(-1) + ik*i;
     y = alfa*k(-1) + (1-alfa)*n + z;
     n = -eta* c + y;
     r = - (alfa/R_bar)*yk_bar*k(-1) + (alfa/R_bar)*yk_bar*y;
     c = c(+1) - (1/eta)* r(+1);
     z = rho* z(-1) + e;
     ynr = y-n+u1;
     nn = n +u2;
     iy = i-y+u3;  
end; 

shocks;
var e; stderr sig_e;

// for exercises 1 and 2 add
var eg; stderr sig_g;

var u1; stderr 0.001;
var u2; stderr 0.001;
var u3; stderr 0.001;
end;

estimated_params;
// PARAM NAME, INITVAL, LB, UB
stderr e, 0.6,0.01,0.9;
stderr u1,0.01,0.001,0.05;
stderr u2,0.01,0.001,0.05;
stderr u3,0.01,0.001,0.05;
rho,0.97,0.2,0.9999;
alfa,0.33,0.2,0.6;
delta,0.025,0.015,0.035;
eta, 1.5,1.0,5.0;
%cy,0.8,0.4,0.9;

// for exercises 1 and 2 add
 rhog, 0.8, 0.2, 0.99;
 stderr eg, 0.5,0.01,0.9;

// for exercises also add 2
% ag, 0.1,-0.3,0.3;
end;


// for exercise 3 use
%estimated_params;
%stderr e, 0.6,0.01,1000.0;
%stderr u1,0.01,0.001,10.0;
%stderr u2,0.01,0.001,10.0;
%stderr u3,0.01,0.001,10.0;
%rho,0.97,-0.9999,0.9999;
%alfa,0.33,0.0,0.9999;
%delta,0.025,0.001,0.999;
%eta, 1.5,1.0,100.0;
%cy,0.8,0.0,0.9999;
%end;

varobs ynr nn iy;

estimation(datafile=RBC_data,mode_compute=5,lik_init=2);

   
