// RBC_model_simul.mod, version 3.2, fabio canova
// RBC model with utility in leisure a-la Hansen (1985)
// This file simulates the model and store the observables in RBC_simuldata.mat
// the .mat file is used in RBC_model_ML for the estimation

close all;clc;

var y c i r k n z ynr nn iy;
 
varexo e u1 u2 u3;  
 
parameters yk_bar R_bar ik cy delta alfa eta rho sig_e;

N_bar     = 1.0/3; % Steady state employment is one fourth of total time endowment
Z_bar     = 1.0;   % Normalization
alfa       = 0.36; % Capital share
delta     = 0.025; % Depreciation rate for capital
R_bar     = 1.01;  % One percent real interest per quarter
eta       = 1.0;   % constant of relative risk aversion = 1/(coeff. of intertemporal substitution)
rho       = 0.95;  % autocorrelation of technology shock
sig_e  = 0.712;  % Standard deviation of technology shock.  Units: Percent.

// Steady states:
betta   = 1.0/R_bar;  % Discount factor beta
yk_bar  = (R_bar + delta - 1)/alfa;  
K_bar   = (yk_bar / Z_bar)^(1.0/(alfa-1)) * N_bar;
I_bar   = delta * K_bar;
Y_bar   = yk_bar * K_bar;
C_bar   = Y_bar - delta*K_bar;
A       =  C_bar^(-eta) * (1 - alfa) * Y_bar/N_bar; % Parameter in utility function
// parameters in the dynamic model  
ik = I_bar/K_bar;
cy = C_bar/Y_bar;
  
% we add three measurement errors to the observables yn (labor productivity) nn (employment), iy (investment 
% to output ratio)  that  is the model becomes
% x(t) = A(theta)x(t-1)+B(theta) e(t)
% y(t) =  Cx(t) + u(t)
% where y(t) are the observable counterparts of the model based quantities x(t)
% and u(t) is a measurement error.

model(linear); 
% model
     y = cy*c+(1-cy)*i;
     k = (1-delta)*k(-1) + ik*i;
     y = alfa*k(-1) + (1-alfa)*n + z;
     n = -eta* c + y;
     r = - (alfa/R_bar)*yk_bar*k(-1) + (alfa/R_bar)*yk_bar*y;
     c = c(+1) - (1/eta)* r(+1);
     z = rho* z(-1) + e;
% observable variables
     ynr = y-n+u1;
     nn = n +u2;
     iy = i-y+u3;  
end; 

shocks;
var e; stderr sig_e;
var u1; stderr 0.7;  % measurement error 1
var u2; stderr 0.7;  % measurement error 2
var u3; stderr 0.7;  % measurement error 3
end;

stoch_simul(periods=200) ynr nn iy ;
 
save RBC_simuldata ynr nn iy
