% takes the data simulated in twocountryApprox.mod and estimates a two country  RBC model with perfect
% capital mobility by ML

var c1 c2 k1 k2 a1 a2 y1 y2;
varexo e1 e2;

parameters gamma delta alpha beta rho;

gamma=1;
delta=.05;
alpha=.3;
beta=.98;
rho=.85;

model;
c1=c2;
exp(c1)^(-gamma) = beta*exp(c1(+1))^(-gamma)*(alpha*exp(a1(+1))*exp(k1)^(alpha-1)+1-delta);
exp(c2)^(-gamma) = beta*exp(c2(+1))^(-gamma)*(alpha*exp(a2(+1))*exp(k2)^(alpha-1)+1-delta);
exp(c1)+exp(c2)+exp(k1)-exp(k1(-1))*(1-delta)+exp(k2)-exp(k2(-1))*(1-delta) = exp(a1)*exp(k1(-1))^alpha+exp(a2)*exp(k2(-1))^alpha;
a1=rho*a1(-1)+e1;
a2=rho*a2(-1)+e2;
exp(y1)=exp(a1)*exp(k1(-1))^alpha;
exp(y2)=exp(a2)*exp(k2(-1))^alpha;
end;

initval;
y1=1.1;
y2=1.1;
k1=2.8;
k2=2.8;
c1=.8;
c2=.8;
a1=0;
a2=0;
e1=0;
e2=0;
end;

shocks;
var e1; stderr .08;
var e2; stderr .08;
end;

 
%ML : use the same command as with bayesian estimation but impose very loose priors

estimated_params;
alpha, normal_pdf, .35, 5.0;
rho, normal_pdf, .8, 5.0;
stderr e1, inv_gamma_pdf, .09, .01;
stderr e2, inv_gamma_pdf, .09, .01;
end;

% this was the original way to write  the problem but it does not work anymore in matlab version above 4.2.4.
%estimated_params;
%alpha, 0.35, 0.10, 0.50;
%rho, 0.8, 0.5, 0.95;
%stderr e1, 0.08, 0.05, 0.1;
%stderr e2, 0.08, 0.05, 0.1;
%end;

% Bayesian estimation

% estimated_params;
% alpha, normal_pdf, .35, .05;
% rho, normal_pdf, .8, .05;
% stderr e1, inv_gamma_pdf, .09, 10;
% stderr e2, inv_gamma_pdf, .09, 10;
% end;

varobs y1 y2;
% ML estimation
estimation(datafile=simu2, order=1, mh_replic=0) y1 y2;

% Bayes
% estimation(datafile=simu2,order=1,mh_replic=1200,mh_jscale=.7,nodiagnostic);

